@@ -57,20 +57,18 @@ except Exception:
     pass
 
 
-_VALID_VERSIONS = ['v2', 'v3']
+_VALID_VERSIONS = ['v3']
 V3_SERVICE_TYPE = 'volumev3'
-V2_SERVICE_TYPE = 'volumev2'
-SERVICE_TYPES = {'2': V2_SERVICE_TYPE,
-                 '3': V3_SERVICE_TYPE}
+SERVICE_TYPES = {'3': V3_SERVICE_TYPE}
 REQ_ID_HEADER = 'X-OpenStack-Request-ID'
 
 # tell keystoneclient that we can ignore the /v1|v2/{project_id} component of
 # the service catalog when doing discovery lookups
-for svc in ('volume', 'volumev2', 'volumev3'):
+for svc in ('volume', 'volumev3'):
     discover.add_catalog_discover_hack(svc, re.compile(r'/v[12]/\w+/?$'), '/')
 
 
-def get_server_version(url, insecure=False, cacert=None):
+def get_server_version(url, insecure=False, cacert=None, cert=None):
     """Queries the server via the naked endpoint and gets version info.
 
     :param url: url of the cinder endpoint
@@ -78,9 +76,15 @@ def get_server_version(url, insecure=False, cacert=None):
                      (https) requests
     :param cacert: Specify a CA bundle file to use in verifying a TLS
                             (https) server certificate
+    :param cert: A client certificate to pass to requests. These are of the
+                 same form as requests expects. Either a single filename
+                 containing both the certificate and key or a tuple containing
+                 the path to the certificate then a path to the key. (optional)
     :returns: APIVersion object for min and max version supported by
               the server
     """
+    # NOTE: we (the client) don't support v2 anymore, but this function
+    # is checking the server version
     min_version = "2.0"
     current_version = "2.0"
 
@@ -115,7 +119,7 @@ def get_server_version(url, insecure=False, cacert=None):
                 verify_cert = cacert
             else:
                 verify_cert = True
-        response = requests.get(version_url, verify=verify_cert)
+        response = requests.get(version_url, verify=verify_cert, cert=cert)
         data = json.loads(response.text)
         versions = data['versions']
         for version in versions:
@@ -124,21 +128,37 @@ def get_server_version(url, insecure=False, cacert=None):
                 current_version = version['version']
                 break
             else:
-                # Set the values, but don't break out the loop here in case v3
-                # comes later
-                min_version = '2.0'
-                current_version = '2.0'
+                # keep looking in case this cloud is running v2 and
+                # we haven't seen v3 yet
+                continue
     except exceptions.ClientException as e:
+        # NOTE: logging the warning but returning the lowest server API version
+        # supported in this OpenStack release is the legacy behavior, so that's
+        # what we do here
+        min_version = '3.0'
+        current_version = '3.0'
         logger.warning("Error in server version query:%s\n"
-                       "Returning APIVersion 2.0", str(e.message))
+                       "Returning APIVersion 3.0", str(e.message))
     return (api_versions.APIVersion(min_version),
             api_versions.APIVersion(current_version))
 
 
-def get_highest_client_server_version(url, insecure=False, cacert=None):
-    """Returns highest supported version by client and server as a string."""
-    min_server, max_server = get_server_version(url, insecure, cacert)
+def get_highest_client_server_version(url, insecure=False,
+                                      cacert=None, cert=None):
+    """Returns highest supported version by client and server as a string.
+
+    :raises: UnsupportedVersion if the maximum supported by the server
+             is less than the minimum supported by the client
+    """
+    min_server, max_server = get_server_version(url, insecure, cacert, cert)
     max_client = api_versions.APIVersion(api_versions.MAX_VERSION)
+    min_client = api_versions.APIVersion(api_versions.MIN_VERSION)
+    if max_server < min_client:
+        msg = _("The maximum version supported by the server (%(srv)s) does "
+                "not meet the minimum version supported by this client "
+                "(%(cli)s)") % {"srv": str(max_server),
+                                "cli": api_versions.MIN_VERSION}
+        raise exceptions.UnsupportedVersion(msg)
     return min(max_server, max_client).get_string()
 
 
@@ -286,7 +306,7 @@ class HTTPClient(object):
                  endpoint_type='publicURL', service_type=None,
                  service_name=None, volume_service_name=None,
                  os_endpoint=None, retries=None,
-                 http_log_debug=False, cacert=None,
+                 http_log_debug=False, cacert=None, cert=None,
                  auth_system='keystone', auth_plugin=None, api_version=None,
                  logger=None, user_domain_name='Default',
                  project_domain_name='Default', global_request_id=None):
@@ -324,7 +344,7 @@ class HTTPClient(object):
         self.timeout = timeout
         self.user_domain_name = user_domain_name
         self.project_domain_name = project_domain_name
-
+        self.cert = cert
         if insecure:
             self.verify_cert = False
         else:
@@ -405,6 +425,7 @@ class HTTPClient(object):
             method,
             url,
             verify=self.verify_cert,
+            cert=self.cert,
             **kwargs)
         self.http_log_resp(resp)
 
@@ -701,7 +722,7 @@ def _construct_http_client(username=None, password=None, project_id=None,
                            os_endpoint=None, retries=None,
                            http_log_debug=False,
                            auth_system='keystone', auth_plugin=None,
-                           cacert=None, tenant_id=None,
+                           cacert=None, cert=None, tenant_id=None,
                            session=None,
                            auth=None, api_version=None,
                            **kwargs):
@@ -741,6 +762,7 @@ def _construct_http_client(username=None, password=None, project_id=None,
                           retries=retries,
                           http_log_debug=http_log_debug,
                           cacert=cacert,
+                          cert=cert,
                           auth_system=auth_system,
                           auth_plugin=auth_plugin,
                           logger=logger,
@@ -762,7 +784,6 @@ def _get_client_class_and_version(version):
 
 def get_client_class(version):
     version_map = {
-        '2': 'cinderclient.v2.client.Client',
         '3': 'cinderclient.v3.client.Client',
     }
     try:
@@ -790,10 +811,6 @@ def discover_extensions(version):
 def _discover_via_python_path():
     for (module_loader, name, ispkg) in pkgutil.iter_modules():
         if name.endswith('cinderclient_ext'):
-            if not hasattr(module_loader, 'load_module'):
-                # Python 2.6 compat: actually get an ImpImporter obj
-                module_loader = module_loader.find_module(name)
-
             module = module_loader.load_module(name)
             yield name, module
 
@@ -838,7 +855,7 @@ def Client(version, *args, **kwargs):
 
     Here ``VERSION`` can be a string or
     ``cinderclient.api_versions.APIVersion`` obj. If you prefer string value,
-    you can use ``2`` (deprecated now) or ``3.X`` (where X is a microversion).
+    you can use ``3`` or ``3.X`` (where X is a microversion).
 
 
     Alternatively, you can create a client instance using the keystoneclient
